@@ -16,7 +16,7 @@ use Storable;
 
 use Devel::Cover::DB;
 
-our $VERSION = "0.53";
+our $VERSION = "0.54";
 our $AUTOLOAD;
 
 sub new
@@ -59,7 +59,9 @@ sub AUTOLOAD
             {
                 my $self = shift;
                 my $file = shift;
-                $file = "" unless defined $file;
+                # print "file: $file, condition: $c\n";
+                # TODO - why no file?
+                return unless defined $file;
                 $self->{f}{$file}{$c}
             }
         };
@@ -70,6 +72,8 @@ sub AUTOLOAD
         {
             my $self = shift;
             my $file = shift;
+            die "Bad file: $func: $file, expecting $self->{file}"
+                unless $file eq $self->{file};
             push @{$self->{f}{$file}{$criterion}}, @_;
         };
     }
@@ -93,6 +97,7 @@ sub set_subroutine
 {
     my $self = shift;
     my ($sub_name, $file, $line) = @{$self}{qw( sub_name file line )} = @_;
+
     $self->{additional} = 0;
     if ($self->reuse($file))
     {
@@ -166,31 +171,44 @@ sub reuse
 sub set_file
 {
     my $self = shift;
-    ($self->{file}) = @_;
+    my ($file) = @_;
+    $self->{file} = $file;
+    $self->digest($file)
 }
 
-sub add_digest
+sub add_digest_xxx
 {
     my $self = shift;
-    my ($file, $run) = @_;
+    my ($file, $digest) = @_;
+    print "Adding $digest for $file\n";
+    $self->{f}{$file}{digest} = $digest;
+    push @{$self->{digests}{$digest}}, $file;
+}
+
+sub digest
+{
+    my $self = shift;
+    my ($file) = @_;
+
+    my $f = $self->{f}{$file};
+    # return $f->{digest} if $f->{digest};
+
+    my $digest;
     if (open my $fh, "<", $file)
     {
         binmode $fh;
-        $run->{digest}{$file} = Digest::MD5->new->addfile($fh)->hexdigest;
-        $self->set_digest($file, $run->{digest}{$file});
+        $digest = Digest::MD5->new->addfile($fh)->hexdigest;
+        # $self->add_digest($file, $digest);
+        # print "Adding $digest for $file\n";
+        $self->{f}{$file}{digest} = $digest;
+        push @{$self->{digests}{$digest}}, $file;
     }
     else
     {
         warn "Devel::Cover: Can't open $file for MD5 digest: $!\n";
         # warn "in ", `pwd`;
     }
-}
-
-sub set_digest
-{
-    my $self = shift;
-    my ($file, $digest) = @_;
-    $self->{f}{$file}{digest} = $digest;
+    $digest
 }
 
 sub get_count
@@ -238,6 +256,7 @@ sub write
         my $df = "$dir/$self->{f}{$file}{digest}";
         # TODO - determine if Structure has changed to save writing it.
         # my $f = $df; my $n = 1; $df = $f . "." . $n++ while -e $df;
+        # print "Writing [$file] to [$df]\n";
         Storable::nstore($self->{f}{$file}, $df); # unless -e $df;
     }
 }
@@ -248,6 +267,7 @@ sub read
     my ($digest) = @_;
     my $file     = "$self->{base}/structure/$digest";
     my $s        = retrieve($file);
+    # print "reading $digest: ", Dumper $s;
     $self->{f}{$s->{file}} = $s;
     $self
 }
@@ -258,7 +278,7 @@ sub read_all
     my $dir = $self->{base};
     $dir .= "/structure";
     opendir D, $dir or return;
-    for my $d (grep $_ !~ /\./, readdir D)
+    for my $d (sort grep $_ !~ /\./, readdir D)
     {
         $self->read($d);
     }
@@ -292,7 +312,7 @@ Huh?
 
 =head1 VERSION
 
-Version 0.53 - 17th April 2005
+Version 0.54 - 13th September 2005
 
 =head1 LICENCE
 
