@@ -10,9 +10,9 @@ package Devel::Cover::Report::Text;
 use strict;
 use warnings;
 
-our $VERSION = "0.53";
+our $VERSION = "0.54";
 
-use Devel::Cover::DB 0.53;
+use Devel::Cover::DB 0.54;
 
 sub print_runs
 {
@@ -42,10 +42,10 @@ sub print_file
     my @args = ("line", "err");
     for my $ann (@{$options->{annotations}})
     {
-        for my $a (0 .. $ann->number_of_annotations - 1)
+        for my $a (0 .. $ann->count - 1)
         {
-            $fmt .= "%-" . $ann->get_width($a) . "s ";
-            push @args, $ann->get_header($a);
+            $fmt .= "%-" . $ann->width($a) . "s ";
+            push @args, $ann->header($a);
         }
     }
     my %cr; @cr{$db->criteria} = $db->criteria_short;
@@ -90,10 +90,11 @@ sub print_file
 
             for my $ann (@{$options->{annotations}})
             {
-                for my $a (0 .. $ann->number_of_annotations - 1)
+                for my $a (0 .. $ann->count - 1)
                 {
-                    push @args, $ann->get_annotation($n, $a);
-                    $error ||= $ann->error($n, $a);
+                    push @args,
+                         substr $ann->text($file, $n, $a), 0, $ann->width($a);
+                    $error ||= $ann->error($file, $n, $a);
                 }
             }
 
@@ -104,7 +105,7 @@ sub print_file
                 my $o = shift @{$criteria{$c}};
                 $more ||= @{$criteria{$c}};
                 my $value = $o
-                    ? ($c =~ /statement|pod|time/)
+                    ? ($c =~ /statement|sub|pod|time/)
                         ? $o->covered
                         : $o->percentage
                     : "";
@@ -303,7 +304,7 @@ Huh?
 
 =head1 VERSION
 
-Version 0.53 - 17th April 2005
+Version 0.54 - 13th September 2005
 
 =head1 LICENCE
 
